## Post Processing Results and Plotting 

# Load packages 
library(dplyr)
library(readr)
library(purrr)
library(sf)
library(ggplot2)
library(countrycode)

## SPATIAL AVERAGES 

# Load the spatial data 
spatial <- readxl::read_excel("spatial_info.xlsx")
spatial <- spatial %>% rename("ISO3" = adm0)

# Load prediction results 
results_flies <- read_csv("prediction_results_flies.csv")
results_disease <- read_csv("prediction_results_disease.csv")

# Merge my spatial information to my predictions 
spatial$CNTR_CODE <- countrycode(spatial$ISO3, origin = "iso3c", destination = "iso2c")
results_flies <- left_join(results_flies, spatial)
results_disease <- left_join(results_disease, spatial)
rm(spatial)

# Define the baseline and comparison periods 
results_disease <- results_disease %>% 
  mutate(time_periods = case_when(year > 1980 & year < 2011 ~ 1, 
                                  year > 2014 & year < 2025 ~ 2))

# Create annual Europe averages 
# results_flies <- results_flies %>%
#   group_by(year) %>%
#   mutate(across(neglectus:tobbi, ~ mean(.x, na.rm = TRUE), .names = "euro_annual_{.col}")) %>%
#   ungroup()
results_disease <- results_disease %>%
  group_by(year) %>%
  mutate(across(cl:vl_fly, ~ mean(.x, na.rm = TRUE), .names = "euro_annual_{.col}")) %>%
  ungroup()

# Create annual country averages 
# results_flies <- results_flies %>%
#   group_by(CNTR_CODE, year) %>%
#   mutate(across(neglectus:tobbi, ~ mean(.x, na.rm = TRUE), .names = "iso2_annual_{.col}")) %>%
#   ungroup()
results_disease <- results_disease %>%
  group_by(CNTR_CODE, year) %>%
  mutate(across(cl:vl_fly, ~ mean(.x, na.rm = TRUE), .names = "iso2_annual_{.col}")) %>%
  ungroup()

# Create annual regional averages 
# results_flies <- results_flies %>%
#   group_by(region_un, year) %>%
#   mutate(across(neglectus:tobbi, ~ mean(.x, na.rm = TRUE), .names = "region_annual_{.col}")) %>%
#   ungroup()
results_disease <- results_disease %>%
  group_by(region_un, year) %>%
  mutate(across(cl:vl_fly, ~ mean(.x, na.rm = TRUE), .names = "region_annual_{.col}")) %>%
  ungroup()

# Create period Europe averages 
# results_flies <- results_flies %>%
#   group_by(time_periods) %>%
#   mutate(across(neglectus:tobbi, ~ mean(.x, na.rm = TRUE), .names = "euro_period_{.col}")) %>%
#   ungroup()
results_disease <- results_disease %>%
  group_by(time_periods) %>%
  mutate(across(cl:vl_fly, ~ mean(.x, na.rm = TRUE), .names = "euro_period_{.col}")) %>%
  ungroup()

# Create period country averages 
# results_flies <- results_flies %>%
#   group_by(CNTR_CODE, time_periods) %>%
#   mutate(across(neglectus:tobbi, ~ mean(.x, na.rm = TRUE), .names = "iso2_period_{.col}")) %>%
#   ungroup()
results_disease <- results_disease %>%
  group_by(CNTR_CODE, time_periods) %>%
  mutate(across(cl:vl_fly, ~ mean(.x, na.rm = TRUE), .names = "iso2_period_{.col}")) %>%
  ungroup()

# Create period ISO2 averages 
# results_flies <- results_flies %>%
#   group_by(NUTS_ID, time_periods) %>%
#   mutate(across(neglectus:tobbi, ~ mean(.x, na.rm = TRUE), .names = "nuts2_period_{.col}")) %>%
#   ungroup()
results_disease <- results_disease %>%
  group_by(NUTS_ID, time_periods) %>%
  mutate(across(cl:vl_fly, ~ mean(.x, na.rm = TRUE), .names = "nuts2_period_{.col}")) %>%
  ungroup()

# Create period regional averages 
# results_flies <- results_flies %>%
#   group_by(region_un, time_periods) %>%
#   mutate(across(neglectus:tobbi, ~ mean(.x, na.rm = TRUE), .names = "region_period_{.col}")) %>%
#   ungroup()
results_disease <- results_disease %>%
  group_by(region_un, time_periods) %>%
  mutate(across(cl:vl_fly, ~ mean(.x, na.rm = TRUE), .names = "region_period_{.col}")) %>%
  ungroup()

# Create a palette 
pastel_palette <- c("#FF8C42", "#FF6F91", "#6A9FB5", "#77DD77", "#C39BD3")

## PLOT TIME-SERIES 

# Plot the Europe annual averages 
region_disease <- results_disease %>% 
  dplyr::select(year, region_un, region_annual_cl, region_annual_vl, region_annual_cl_fly, region_annual_vl_fly) %>% 
  distinct() %>% tidyr::gather(model, predicted_risk, 3:6)
region_disease$model <- gsub("region_annual_", "", region_disease$model)
region_disease <- na.omit(region_disease)

region_disease %>% filter(model == "vl_fly" | model == "cl_fly") %>% 
ggplot(aes(x = year, y = predicted_risk, color = model)) + geom_line() + 
  theme_minimal() + labs(x = "Year", y = "Predicted Leishmaniasis Risk (0-1)") +
  geom_text(data = region_disease %>% filter(model == "vl_fly" | model == "cl_fly") %>%
              group_by(model) %>% 
              filter(year == max(year)) %>% 
              mutate(model_label = ifelse(model == "vl_fly", "VL", "CL")),
            aes(label = model_label, x = year, y = predicted_risk),
            hjust = -0.1, vjust = 0, size = 4, fontface = "bold", check_overlap = TRUE
  ) + xlim(1970, 2027) + scale_color_manual(values = pastel_palette) + 
  theme(legend.position = "none", text = element_text(face = "bold"))

# Plot the regional annual averages 
nuts2_flies <- results_disease %>% 
  dplyr::select(year, NUTS_ID, nuts) %>% 
  distinct() %>% tidyr::gather(vector, predicted_risk, 3:7)
region_flies$vector <- gsub("region_annual_", "", region_flies$vector)
region_flies <- na.omit(region_flies)

region_disease %>% filter(model == "vl_fly" | model == "cl_fly") %>% 
  mutate(model_label = ifelse(model == "vl_fly", "VL", "CL")) %>%
ggplot(aes(x = year, y = predicted_risk, color = region_un)) + geom_line() + 
  theme_minimal() + labs(x = "Year", y = "Predicted Leishmaniasis Risk (0-1)", color = "Regions") +
  scale_color_manual(values = pastel_palette) + facet_wrap(~model_label) +
  theme(text = element_text(face = "bold"),
        legend.position =  "top",
        strip.text = element_text(size = 12, face = "bold"),
        legend.key.height = unit(1.5, "lines"),  # Adjust height
        legend.key.width = unit(1.5, "lines"),   # Adjust width
        legend.text = element_text(size = 10)) 

# Save the averages
country_flies <- results_flies %>% 
  dplyr::select(year, CNTR_CODE, iso2_annual_neglectus, iso2_annual_perfiliewi, iso2_annual_ariasi, iso2_annual_perniciosus, iso2_annual_tobbi) %>% 
  distinct() %>% tidyr::gather(vector, predicted_risk, 3:7)
country_flies$vector <- gsub("iso2_annual_", "", country_flies$vector)

nuts2_flies <- results_flies %>% 
  dplyr::select(year, NUTS_ID, neglectus, perfiliewi, ariasi, perniciosus, tobbi) %>% 
  distinct() 

fly_list <- list(euro_flies, region_flies, country_flies, nuts2_flies)

openxlsx::write.xlsx(fly_list, file = "annual_averages.xlsx")

## PLOT SPATIAL CHANGES 

nuts2_disease <- results_disease %>%
  dplyr::select(time_periods, NUTS_ID, nuts2_period_cl_fly, nuts2_period_vl_fly) 
nuts2_disease <- na.omit(nuts2_disease) 
nuts2_disease <- nuts2_disease %>% distinct()

differences <- nuts2_disease %>%
  group_by(NUTS_ID) %>%
  summarize(across(starts_with("nuts2_period_"), 
                   ~ round(((.[time_periods == 2] - .[time_periods == 1]) / .[time_periods == 1]) * 100, 2))) %>%
  mutate(time_periods = "Percentage Change")
differences <- differences %>% tidyr::gather(disease, percent_change_predicted_risk, 2:3)
differences$disease <- gsub("nuts2_period_", "", differences$disease)
differences <- differences %>% mutate(disease = if_else(disease == "cl_fly", "CL", "VL"))

# Plot as a bar
ggplot(differences, aes(x = reorder(vector, percent_change_predicted_risk), 
                        y = percent_change_predicted_risk, color = region_un, fill = region_un)) + 
  geom_bar(stat = "identity", position = "dodge") + scale_color_manual(values = pastel_palette) + 
  scale_fill_manual(values = pastel_palette) + 
  theme_minimal() + coord_flip() + 
  theme(text = element_text(face = "bold"),
        legend.position = c(0.75, 0.25), 
        legend.justification = "left",
        strip.text = element_text(size = 12, face = "bold"),
        legend.key.height = unit(1.5, "lines"),  # Adjust height
        legend.key.width = unit(1.5, "lines"),   # Adjust width
        legend.text = element_text(size = 10)) +
  labs(y = "Change in Predicted Sandfly Risk (%)", x = "Vector Species", color = "Regions", fill = "Regions")
                                                          
# Plot in a map
nuts <- read_sf("NUTS_RG_20M_2024_4326.shp", stringsAsFactors = F)
nuts0 <- nuts %>% filter(LEVL_CODE == 0)
nuts2 <- nuts %>% filter(LEVL_CODE == 2)

nuts2 <- left_join(nuts2, differences)
nuts2 <- nuts2 %>% filter(!is.na(percent_change_predicted_risk))
nuts2$percent_change_predicted_risk <- as.numeric(nuts2$percent_change_predicted_risk)
nuts2 <- nuts2 %>%
  mutate(percent_change_predicted_risk = ifelse(percent_change_predicted_risk > 100, 100, percent_change_predicted_risk))

ggplot(nuts2) + 
  geom_sf(aes(fill = percent_change_predicted_risk)) + 
  theme_minimal() + 
  scale_fill_gradient2(
    low = "#377EB8",   # Blue for negative values
    mid = "white",     # White for neutral (0)
    high = "#E41A1C",  # Red for positive values
    midpoint = 0,      # Set the midpoint to zero
    limits = c(-100, 100)  # Adjust limits to your data range
  ) +
  facet_wrap(~disease) + 
  xlim(-30, 50) + 
  ylim(30, 80) + 
  theme(
    text = element_text(face = "bold"),
    legend.position = "top",
    strip.text = element_text(size = 12, face = "bold"),
    legend.key.height = unit(1.5, "lines"),  
    legend.key.width = unit(1.5, "lines"),   
    legend.text = element_text(size = 10)
  ) + labs(fill = "Change in Predicted Leishmaniasis Risk (%)")


